pub unsafe trait TransmuteFrom<Src: ?Sized, SV, DV> { }Expand description
Denotes that any SV-valid Src may soundly be transmuted into a
DV-valid Self.
Safety
Given src: Ptr<Src, (_, _, SV)> and dst: Ptr<Dst, (_, _, DV)>, if the
referents of src and dst are the same size, then the set of bit patterns
allowed to appear in src’s referent must be a subset of the set allowed to
appear in dst’s referent.
If the referents are not the same size, then Dst: TransmuteFrom<Src, SV, DV> conveys no safety guarantee.