InvariantsEq

Trait InvariantsEq 

pub unsafe trait InvariantsEq<T: ?Sized> { }
Denotes that two types have the same invariants.

Safety

It is sound for safe code to operate on a &T and a &Self pointing to the same referent at the same time - no such safe code can cause undefined behavior.

Implementors