pub unsafe trait MutationCompatible<Src: ?Sized, A: Aliasing, SV, DV, R> { }Expand description
Denotes that src: Ptr<Src, (A, _, SV)> and dst: Ptr<Self, (A, _, DV)>,
referencing the same referent at the same time, cannot be used by safe code
to break library safety invariants of Src or Self.
Safety
At least one of the following must hold:
Src: Read<A, _>andSelf: Read<A, _>Self: InvariantsEq<Src>, and, for someV:Dst: TransmuteFrom<Src, V, V>Src: TransmuteFrom<Dst, V, V>